# -*- coding: utf-8 -*-
"""
    mackerel.host
    ~~~~~~~~~~~~~

    Mackerel client implemented by Python.

    Ported from `mackerel-client-ruby`.
    <https://github.com/mackerelio/mackerel-client-ruby>

    :copyright: (c) 2014 Hatena, All rights reserved.
    :copyright: (c) 2015 Shinya Ohyanagi, All rights reserved.
    :copyright: (c) 2016 Iskandar Setiadi, All rights reserved.
    :license: BSD, see LICENSE for more details.
"""
import re


class Host(object):
    MACKEREL_INTERFACE_NAME_PATTERN = re.compile(r'^eth\d')

    def __init__(self, **kwargs):
        """Construct a host.

        :param name: Host name
        :param meta: Host metadata
        :param type: Host type
        :param status: Host status
        :param memo: Host memo
        :param is_retired: Retired flag
        :param id: Host id
        :param created_at: Created datetime
        :param roles: Host roles
        :param interfaces: Host interfaces
        """
        self.args = kwargs
        self.name = kwargs.get('name', None)
        self.meta = kwargs.get('meta', None)
        self.type = kwargs.get('type', None)
        self.status = kwargs.get('status', None)
        self.memo = kwargs.get('memo', None)
        self.is_retired = kwargs.get('isRetired', None)
        self.id = kwargs.get('id', None)
        self.created_at = kwargs.get('createdAt', None)
        self.roles = kwargs.get('roles', None)
        self.interfaces = kwargs.get('interfaces', None)

    def ip_addr(self):
        """Get ipaddress."""
        for i in self.interfaces:
            if self.MACKEREL_INTERFACE_NAME_PATTERN.search(i['name']):
                return i['ipAddress']

    def mac_addr(self):
        """Get MAC address."""
        for i in self.interfaces:
            if self.MACKEREL_INTERFACE_NAME_PATTERN.search(i['name']):
                return i['macAddress']

    def __repr__(self):
        repr = '<Host('
        repr += 'name={0}, meta={1}, type={2}, status={3}, memo={4},'
        repr += 'is_retired={5}, id={6}, created_at={7}, roles={8},'
        repr += 'interfaces={9})'
        return repr.format(self.name, self.meta, self.type, self.status,
                           self.memo, self.is_retired, self.id,
                           self.created_at, self.roles, self.interfaces)



# -*- coding: utf-8 -*-
"""
    mackerel.clienthde
    ~~~~~~~~~~~~~~~~~~~

    Mackerel client implemented by Python.


    :copyright: (c) 2015 Shinya Ohyanagi, All rights reserved.
    :copyright: (c) 2016 Iskandar Setiadi, All rights reserved.
    :license: BSD, see LICENSE for more details.
"""
import os
from setuptools import setup, find_packages

requires = ['requests', 'simplejson', 'click']

app_name = 'mackerel.clienthde'

rst_path = os.path.join(os.path.dirname(__file__), 'README.rst')
description = ''
with open(rst_path) as f:
    description = f.read()

setup(
    name=app_name,
    version='0.0.1',
    author='Shinya Ohyanagi, Iskandar Setiadi',
    author_email='iskandarsetiadi@gmail.com',
    url='https://github.com/HDE/py-mackerel-client',
    description='Mackerel client implemented by Python.',
    long_description=description,
    license='BSD',
    platforms='any',
    packages=find_packages(exclude=['tests']),
    package_dir={'': '.'},
    install_requires=requires,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python'
    ],
    entry_points="""
    [console_scripts]
    mkr.py = mackerel.runner:main
    """,
    tests_require=['requests', 'simplejson', 'mock'],
    test_suite='tests'
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
    mackerel.runner
    ~~~~~~~~~~~~~~~

    Mackerel client implemented by Python.

    Ported from `mackerel-client-ruby`.
    <https://github.com/mackerelio/mackerel-client-ruby>

    :copyright: (c) 2015 Shinya Ohyanagi, All rights reserved.
    :copyright: (c) 2016 Iskandar Setiadi, All rights reserved.
    :license: BSD, see LICENSE for more details.
"""
import os
import logging
import click
from mackerel.clienthde import Client


class Runner(object):

    def __init__(self, verbose=False):
        api_key = os.environ.get('MACKEREL_APIKEY')
        logger = None
        level = logging.INFO
        if verbose is True:
            level = logging.DEBUG

        logging.getLogger('requests').level = logging.ERROR
        logging.basicConfig(level=level, format=Client.debug_log_format)
        logger = logging.getLogger('mackerel.client')

        self.client = Client(mackerel_api_key=api_key, logger=logger)


@click.group()
@click.option('--verbose', default=False, type=click.BOOL)
@click.pass_context
def cli(ctx, verbose):
    runner = Runner(verbose)
    ctx.obj['instance'] = runner


@cli.command()
@click.option('--host-id', default=None)
@click.option('--status', default=None)
@click.pass_context
def status(ctx, host_id, status):
    runner = ctx.obj['instance']
    if status is None:
        if host_id is None:
            data = runner.client.get_hosts()
        else:
            data = [runner.client.get_host(host_id)]
        message = 'name: {0} status: {1} id: {2} roles: {3}'
        for d in data:
            click.echo(message.format(d.name, d.status, d.id, d.roles))
    else:
        runner.client.update_host_status(host_id, status)


@cli.command()
@click.option('--name', default=None)
@click.option('--service', default=None)
@click.option('--role', default=None)
@click.pass_context
def info(ctx, name, service, role):
    runner = ctx.obj['instance']
    data = runner.client.get_hosts(name=name, service=service, role=role)
    [click.echo(host.status) for host in data]


@cli.command()
@click.option('--host-id')
@click.pass_context
def retire(ctx, host_id):
    runner = ctx.obj['instance']
    runner.client.retire_host(host_id)


def main():
    cli(obj={})


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
"""
    mackerel.client
    ~~~~~~~~~~~~~~~

    Mackerel client implemented by Python.

    Ported from `mackerel-client-ruby`.
    <https://github.com/mackerelio/mackerel-client-ruby>

    :copyright: (c) 2014 Hatena, All rights reserved.
    :copyright: (c) 2015 Shinya Ohyanagi, All rights reserved.
    :copyright: (c) 2016 Iskandar Setiadi, All rights reserved.
    :license: BSD, see LICENSE for more details.
"""
import logging
import requests
import simplejson as json
from mackerel.host import Host


class MackerelClientError(Exception):
    pass


class Client(object):

    #: Mackerel apikey error message.
    ERROR_MESSAGE_FOR_API_KEY_ABSENCE = 'API key is absent. Set your API key in a environment variable called MACKEREL_APIKEY.'
    #: Log format.
    debug_log_format = (
        '[%(asctime)s %(levelname)s][%(pathname)s:%(lineno)d]: %(message)s'
    )

    def __init__(self, logger=None, **kwargs):
        """Construct a Mackerel client.

        :param mackerel_origin: API endpoint
        :param mackerel_api_key: API key
        """
        self.origin = kwargs.get('mackerel_origin', 'https://mackerel.io')
        api_key = kwargs.get('mackerel_api_key', None)
        if api_key is None:
            raise MackerelClientError(self.ERROR_MESSAGE_FOR_API_KEY_ABSENCE)

        self.api_key = api_key
        if logger is None:
            logging.basicConfig(level=logging.INFO,
                                format=self.debug_log_format)
            self.logger = logging.getLogger('mackerel.client')
        else:
            self.logger = logger

    def get_host(self, host_id):
        """Get registered host.

        :param host_id: Host id
        """
        uri = '/api/v0/hosts/{0}'.format(host_id)
        data = self._request(uri)

        return Host(**data['host'])

    def update_host_status(self, host_id, status):
        """Update host status.

        :param host_id: Host id
        :param status: `standby`, `working`, `maintenance` or `poweroff`
        """
        if status not in ['standby', 'working', 'maintenance', 'poweroff']:
            raise MackerelClientError('no such status: {0}'.format(status))

        uri = '/api/v0/hosts/{0}/status'.format(host_id)
        headers = {'Content-Type': 'application/json'}
        params = json.dumps({'status': status})
        data = self._request(uri, method='POST', headers=headers, params=params)

        return data

    def retire_host(self, host_id):
        """Retire host.

        :param host_id: Host id
        """
        uri = '/api/v0/hosts/{0}/retire'.format(host_id)
        headers = {'Content-Type': 'application/json'}
        data = self._request(uri, method='POST', headers=headers)

        return data

    def post_metrics(self, metrics):
        """Post metrics.

        :param metrics: Metrics
        """
        uri = '/api/v0/tsdb'
        headers = {'Content-Type': 'application/json'}
        params = json.dumps(metrics)
        data = self._request(uri, method='POST', headers=headers, params=params)

        return data

    def get_latest_metrics(self, host_ids, names):
        """Get latest metrics.

        :param host_ids: Host id list
        :param names: Metrics list
        """
        hosts_query = '&'.join(['hostId={0}'.format(id) for id in host_ids])
        names_query = '&'.join(['name={0}'.format(name) for name in names])
        uri = '/api/v0/tsdb/latest?{0}&{1}'.format(hosts_query, names_query)

        data = self._request(uri)

        return data

    def post_service_metrics(self, service_name, metrics):
        """Post service metrics.

        :param service_name: Registered service name
        :param metrics: Metrics list
        """
        uri = '/api/v0/services/{0}/tsdb'.format(service_name)
        headers = {'Content-Type': 'application/json'}
        params = json.dumps(metrics)
        data = self._request(uri, method='POST', headers=headers, params=params)

        return data

    def get_hosts(self, **kwargs):
        """Get hosts.

        :param service: Service name
        :param roles: Service role
        :param name: Host name
        """
        uri = '/api/v0/hosts.json'
        params = {}
        if kwargs.get('service', None):
            params['service'] = kwargs.get('service')

        if kwargs.get('roles', None):
            params['roles'] = kwargs.get('roles')

        if kwargs.get('name', None):
            params['name'] = kwargs.get('name')

        hosts = self._request(uri, params=params)
        return [Host(**host) for host in hosts['hosts']]

    def _request(self, uri, method='GET', headers=None, params=None):
        """Request to mackerel.

        :param uri: Request uri
        :param method: HTTP Method
        :param headers: HTTP Headers
        :param params: HTTP Body or params
        """
        uri = '{0}{1}'.format(self.origin, uri)
        if headers is None:
            headers = {'X-Api-Key': self.api_key}
        else:
            headers.update({'X-Api-Key': self.api_key})

        self.logger.debug('{0} {1} {2}'.format(method, uri, params))
        if method == 'GET':
            res = requests.get(uri, headers=headers, params=params)
        elif method == 'POST':
            res = requests.post(uri, headers=headers, data=params)
        else:
            message = '{0} is not supported.'.format(method)
            raise NotImplementedError(message)

        self.logger.debug('Response from {0} is {1}'.format(self.origin,
                                                            res.status_code))
        if res.status_code != 200:
            message = '{0} {1} failed: {2}'.format(method, uri, res.status_code)
            raise MackerelClientError(message)

        data = json.loads(res.content)

        return data
